"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withAlphaValue = withAlphaValue;
exports.default = withAlphaVariable;

var culori = _interopRequireWildcard(require("culori"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function isValidColor(color) {
  return culori.parse(color) !== undefined;
}

function withAlphaValue(color, alphaValue, defaultValue) {
  if (typeof color === 'function') {
    return color({
      opacityValue: alphaValue
    });
  }

  if (isValidColor(color)) {
    // Parse color
    const parsed = culori.parse(color); // Apply alpha value

    parsed.alpha = alphaValue; // Format string

    let value;

    if (parsed.mode === 'hsl') {
      value = culori.formatHsl(parsed);
    } else {
      value = culori.formatRgb(parsed);
    } // Correctly apply CSS variable alpha value


    if (typeof alphaValue === 'string' && alphaValue.startsWith('var(') && value.endsWith('NaN)')) {
      value = value.replace('NaN)', `${alphaValue})`);
    } // Color could not be formatted correctly


    if (!value.includes('NaN')) {
      return value;
    }
  }

  return defaultValue;
}

function withAlphaVariable({
  color,
  property,
  variable
}) {
  if (typeof color === 'function') {
    return {
      [variable]: '1',
      [property]: color({
        opacityVariable: variable,
        opacityValue: `var(${variable})`
      })
    };
  }

  if (isValidColor(color)) {
    const parsed = culori.parse(color);

    if ('alpha' in parsed) {
      // Has an alpha value, return color as-is
      return {
        [property]: color
      };
    }

    const formatFn = parsed.mode === 'hsl' ? 'formatHsl' : 'formatRgb';
    const value = culori[formatFn]({ ...parsed,
      alpha: NaN // intentionally set to `NaN` for replacing

    }).replace('NaN)', `var(${variable}))`);
    return {
      [variable]: '1',
      [property]: value
    };
  }

  return {
    [property]: color
  };
}